import pytest
from pytest_httpx import HTTPXMock

from smsru import AioSMSru, SMSru


@pytest.mark.asyncio
async def test_aio_senders(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/senders?api_id=apfel&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "senders": ["pfelservice", "pfelstore"],
        },
    )
    client = AioSMSru("apfel")
    response = await client.senders()
    assert response.status_code == 100
    assert "pfelservice" in response.senders
    assert "pfelstore" in response.senders
    assert response.status == "OK"


def test_senders(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/senders?api_id=apfel&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "senders": ["pfelservice", "pfelstore"],
        },
    )
    client = SMSru("apfel")
    response = client.senders()
    assert response.status_code == 100
    assert "pfelservice" in response.senders
    assert "pfelstore" in response.senders
    assert response.status == "OK"



from typing import Optional

from pydantic import BaseModel


class SMSruBase(BaseModel):
    status: str
    status_code: int
    status_text: Optional[str]


class SMSruWithBalance(SMSruBase):
    balance: float



from typing import Optional

from . import SMSruBase


class SMSruLimit(SMSruBase):
    total_limit: Optional[int]
    used_today: Optional[int]


class SMSruFreeLimit(SMSruBase):
    total_free: Optional[int]
    used_today: Optional[int]



import pytest
from pytest_httpx import HTTPXMock

from smsru import AioSMSru, SMSru


@pytest.mark.asyncio
async def test_aio_limits(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/limit?api_id=apfel&json=1&partner_id=331687",
        json={"status": "OK", "status_code": 100, "total_limit": 100, "used_today": 7},
    )
    client = AioSMSru("apfel")
    response = await client.limit()
    assert response.status_code == 100
    assert response.total_limit == 100
    assert response.used_today == 7


def test_limits(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/limit?api_id=apfel&json=1&partner_id=331687",
        json={"status": "OK", "status_code": 100, "total_limit": 100, "used_today": 7},
    )
    client = SMSru("apfel")
    response = client.limit()
    assert response.status_code == 100
    assert response.total_limit == 100
    assert response.used_today == 7


@pytest.mark.asyncio
async def test_aio_free_limits(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/free?api_id=apfel&json=1&partner_id=331687",
        json={"status": "OK", "status_code": 100, "total_free": 100, "used_today": 7},
    )
    client = AioSMSru("apfel")
    response = await client.free_limit()
    assert response.status_code == 100
    assert response.total_free == 100
    assert response.used_today == 7


def test_free_limits(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/free?api_id=apfel&json=1&partner_id=331687",
        json={"status": "OK", "status_code": 100, "total_free": 100, "used_today": 7},
    )
    client = SMSru("apfel")
    response = client.free_limit()
    assert response.status_code == 100
    assert response.total_free == 100
    assert response.used_today == 7



import pytest
from pytest_httpx import HTTPXMock

from smsru import AioSMSru, SMSru


@pytest.mark.asyncio
async def test_aio_check_sms(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/status?api_id=apfel&sms_id=000000-000001,000000-000002,000000-000003&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "000000-000001": {
                    "status": "OK",
                    "status_code": 103,
                    "cost": 0.50,
                    "status_text": "Сообщение доставлено",
                },
                "000000-000002": {
                    "status": "OK",
                    "status_code": 104,
                    "cost": 0.50,
                    "status_text": "Не может быть доставлено: время жизни истекло",
                },
                "000000-000003": {
                    "status": "ERROR",
                    "status_code": -1,
                    "status_text": "Сообщение не найдено",
                },
            },
            "balance": 4122.56,
        },
    )
    client = AioSMSru("apfel")
    response = await client.check_sms(
        ["000000-000001", "000000-000002", "000000-000003"]
    )
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["000000-000001"].cost == 0.5
    assert response.sms["000000-000003"].status_code == -1
    assert response.sms["000000-000003"].cost is None
    assert response.balance == 4122.56


@pytest.mark.asyncio
async def test_aio_check_one_sms(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/status?api_id=apfel&sms_id=000000-000001&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "000000-000001": {
                    "status": "OK",
                    "status_code": 103,
                    "cost": 0.50,
                    "status_text": "Сообщение доставлено",
                }
            },
            "balance": 4122.56,
        },
    )
    client = AioSMSru("apfel")
    response = await client.check_sms("000000-000001")
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["000000-000001"].cost == 0.5
    assert response.balance == 4122.56


def test_check_sms(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/status?api_id=apfel&sms_id=000000-000001,000000-000002,000000-000003&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "000000-000001": {
                    "status": "OK",
                    "status_code": 103,
                    "cost": 0.50,
                    "status_text": "Сообщение доставлено",
                },
                "000000-000002": {
                    "status": "OK",
                    "status_code": 104,
                    "cost": 0.50,
                    "status_text": "Не может быть доставлено: время жизни истекло",
                },
                "000000-000003": {
                    "status": "ERROR",
                    "status_code": -1,
                    "status_text": "Сообщение не найдено",
                },
            },
            "balance": 4122.56,
        },
    )
    client = SMSru("apfel")
    response = client.check_sms(["000000-000001", "000000-000002", "000000-000003"])
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["000000-000001"].cost == 0.5
    assert response.sms["000000-000003"].status_code == -1
    assert response.sms["000000-000003"].cost is None
    assert response.balance == 4122.56


def test_check_one_sms(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/status?api_id=apfel&sms_id=000000-000001&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "000000-000001": {
                    "status": "OK",
                    "status_code": 103,
                    "cost": 0.50,
                    "status_text": "Сообщение доставлено",
                }
            },
            "balance": 4122.56,
        },
    )
    client = SMSru("apfel")
    response = client.check_sms("000000-000001")
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["000000-000001"].cost == 0.5
    assert response.balance == 4122.56



from typing import Optional, Dict

from . import SMSruWithBalance, SMSruBase


class SMSruSendSms(SMSruBase):
    sms_id: Optional[str]


class SMSruSendSmsResponse(SMSruWithBalance):
    sms: Dict[str, SMSruSendSms]


class SMSruCheckSms(SMSruBase):
    cost: Optional[float]


class SMSruCheckSmsResponse(SMSruWithBalance):
    sms: Dict[str, SMSruCheckSms]


class SMSruSmsCost(SMSruBase):
    cost: Optional[float]
    sms: Optional[int]


class SMSruSmsCostResponse(SMSruBase):
    sms: Dict[str, SMSruSmsCost]
    total_cost: float
    total_sms: int



from os import environ

from setuptools import find_packages, setup

classifiers = [
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Intended Audience :: Developers",
    "Topic :: Software Development :: Libraries",
]

with open("README.md", "r") as f:
    readme = f.read()


def requirements():
    with open("requirements.txt", "r") as req:
        return [r for r in req.read().split("\n") if r]


setup(
    name="aiosmsru",
    version=environ.get("TAG_VERSION").replace("v", ""),
    author="WhiteApfel",
    author_email="white@pfel.ru",
    url="https://github.com/WhiteApfel/aiosmsru",
    packages=find_packages("."),
    description="(A)sync client for sms.ru with pydantic responses",
    long_description=readme,
    long_description_content_type="text/markdown",
    license="Mozilla Public License 2.0",
    classifiers=classifiers,
    keywords="sms wrapper api russia",
    install_requires=requirements(),
)



import pytest
from pytest_httpx import HTTPXMock

from smsru import AioSMSru, SMSru


@pytest.mark.asyncio
async def test_aio_send_one_sms(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/send?api_id=apfel&to=79991398805&msg=hello+world&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "79991398805": {
                    "status": "OK",
                    "status_code": 100,
                    "sms_id": "000000-10000000",
                }
            },
            "balance": 4122.56,
        },
    )
    client = AioSMSru("apfel")
    response = await client.send_sms("79991398805", "hello world")
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["79991398805"].sms_id == "000000-10000000"
    assert response.balance == 4122.56


def test_send_one_sms(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/send?api_id=apfel&to=79991398805&msg=hello+world&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "79991398805": {
                    "status": "OK",
                    "status_code": 100,
                    "sms_id": "000000-10000000",
                }
            },
            "balance": 4122.56,
        },
    )
    client = SMSru("apfel")
    response = client.send_sms("79991398805", "hello world")
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["79991398805"].sms_id == "000000-10000000"
    assert response.balance == 4122.56


@pytest.mark.asyncio
async def test_aio_send_any_sms_one_text(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/send?api_id=apfel&to=79991398805,79956896018&msg=hello+world&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "79991398805": {
                    "status": "OK",
                    "status_code": 100,
                    "sms_id": "000000-10000000",
                },
                "79956896018": {
                    "status": "ERROR",
                    "status_code": 207,
                    "status_text": "По небритому анусу пошло всё",
                },
            },
            "balance": 4122.56,
        },
    )
    client = AioSMSru("apfel")
    response = await client.send_sms(["79991398805", "79956896018"], "hello world")
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["79991398805"].sms_id == "000000-10000000"
    assert response.sms["79956896018"].status_code == 207
    assert response.balance == 4122.56


def test_aio_send_any_sms(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/sms/send?api_id=apfel&to[79991398805]=hello+world&to[79956896018]=hello+world&json=1&partner_id=331687",
        json={
            "status": "OK",
            "status_code": 100,
            "sms": {
                "79991398805": {
                    "status": "OK",
                    "status_code": 100,
                    "sms_id": "000000-10000000",
                },
                "79956896018": {
                    "status": "ERROR",
                    "status_code": 207,
                    "status_text": "По небритому анусу пошло всё",
                },
            },
            "balance": 4122.56,
        },
    )
    client = SMSru("apfel")
    response = client.send_sms(
        ["79991398805", "79956896018"], ["hello world", "hello world"]
    )
    assert response.status_code == 100
    assert response.status == "OK"
    assert response.sms["79991398805"].sms_id == "000000-10000000"
    assert response.sms["79956896018"].status_code == 207
    assert response.balance == 4122.56



import pytest
from pytest_httpx import HTTPXMock

from smsru import AioSMSru, SMSru


@pytest.mark.asyncio
async def test_aio_balance(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/balance?api_id=apfel&json=1&partner_id=331687",
        json={"status": "OK", "status_code": 100, "balance": 4762.58},
    )
    client = AioSMSru("apfel")
    response = await client.balance()
    assert response.status_code == 100
    assert response.balance == 4762.58
    assert response.status == "OK"


def test_balance(httpx_mock: HTTPXMock):
    httpx_mock.add_response(
        url="https://sms.ru/my/balance?api_id=apfel&json=1&partner_id=331687",
        json={"status": "OK", "status_code": 100, "balance": 4762.58},
    )
    client = SMSru("apfel")
    response = client.balance()
    assert response.status_code == 100
    assert response.balance == 4762.58
    assert response.status == "OK"



from typing import Optional, Union
import sys

from smsru.models import SMSruWithBalance, SMSruBase
from smsru.models.limits import SMSruLimit, SMSruFreeLimit
from smsru.models.senders import SMSruSenders
from smsru.models.stoplist import SMSruStoplist

if sys.version_info >= (3, 9):
    from collections.abc import Sequence
else:
    from typing import Sequence
from httpx import AsyncClient, Response

from smsru.models.sms import (
    SMSruSendSmsResponse,
    SMSruCheckSmsResponse,
    SMSruSmsCostResponse,
)


class AioSMSru:
    def __init__(self, api_id: str, domain: str = "sms.ru"):
        self._api_id = api_id
        self._domain = domain
        self._client: Optional[AsyncClient] = None

    @property
    def client(self) -> AsyncClient:
        if not self._client:
            self._client = AsyncClient()
        return self._client

    async def _request(self, endpoint: str, params: dict = None) -> Response:
        if not params:
            params = {}
        params.update({"api_id": self._api_id, "json": 1, "partner_id": 331687})
        response = await self.client.get(
            f"https://{self._domain}/{endpoint}", params=params
        )
        return response

    async def send_sms(
        self,
        recipients: Union[str, Sequence[str]],
        messages: Union[str, Sequence[str]],
        **kwargs,
    ) -> SMSruSendSmsResponse:
        """
        Method for sending SMS.
        Method can send one or more messages
        with the same or different text.

        For example, you can

        :param recipients: Phone number or their sequence
        :type recipients: ``str`` or sequence of ``str``
        :param messages: Message or their sequence according
        to the sequence of recipients
        :param kwargs: ``str`` or sequence of ``str``
        :return: sms.ru response
        :rtype: SMSruSendSmsResponse
        """
        params = {}
        if (  # More than one recipient
            isinstance(recipients, Sequence)
            and not isinstance(recipients, str)
            and len(recipients) > 1
        ):
            if (  # More than one message
                isinstance(messages, Sequence)
                and not isinstance(messages, str)
                and len(messages) > 1
            ):
                if len(recipients) == len(messages):
                    for i, recipient in enumerate(recipients):
                        params[f"to[{recipient}]"] = messages[i]
                else:
                    raise ValueError(
                        "Sequences of recipients and messages "
                        "must be of the same length"
                    )
            elif (  # One message for all recipients
                isinstance(messages, str)
                or isinstance(messages, Sequence)
                and len(messages) == 1
            ):
                if isinstance(messages, Sequence) and not isinstance(messages, str):
                    messages = messages[0]
                params["to"] = ",".join(recipients)
                params["msg"] = messages
            else:
                raise ValueError(
                    "Messages must be str (message) " "or sequence of str (messages)"
                )
        elif (  # Only one recipient
            isinstance(recipients, str)
            or isinstance(recipients, Sequence)
            and len(recipients) == 1
        ):
            if (
                isinstance(messages, str)
                or isinstance(messages, Sequence)
                and len(messages) == 1
            ):
                if isinstance(messages, Sequence) and not isinstance(messages, str):
                    messages = messages[0]
                params["to"] = recipients
                params["msg"] = messages
            else:
                raise ValueError(
                    "If there is only one recipient, "
                    "then there can be no more than one message"
                )
        else:
            raise ValueError(
                "Recipients must be str (phone number) "
                "or sequence of str (phone numbers)"
            )

        params.update(kwargs)

        response = await self._request("sms/send", params)

        return SMSruSendSmsResponse(**response.json())

    async def check_sms(self, ids: Union[str, Sequence[str]]) -> SMSruCheckSmsResponse:
        """
        Check the status of one or more messages

        :param ids: message identifier or sequence
        :return: sms.ru response
        :rtype: SMSruCheckSmsResponse
        """
        if isinstance(ids, str):
            ids = [ids]
        if not isinstance(ids, Sequence):
            raise ValueError("ids must be str or sequence of str")

        params = {"sms_id": ",".join(ids)}

        response = await self._request("sms/status", params)

        return SMSruCheckSmsResponse(**response.json())

    async def sms_cost(
        self,
        recipients: Union[str, Sequence[str]],
        messages: Union[str, Sequence[str]],
        **kwargs,
    ) -> SMSruSmsCostResponse:
        """
        Get the cost of sending a specific message to a specific number

        :param recipients: Phone number or their sequence
        :type recipients: ``str`` or sequence of ``str``
        :param messages: Message or their sequence according
        to the sequence of recipients
        :type messages: ``str`` or sequence of ``str``
        :param kwargs: Other parameters according to the documentation sms.ru
        :return: sms.ru response
        :rtype: SMSruSmsCostResponse
        """
        params = {}
        if (  # More than one recipient
            isinstance(recipients, Sequence)
            and not isinstance(recipients, str)
            and len(recipients) > 1
        ):
            if (  # More than one message
                isinstance(messages, Sequence)
                and not isinstance(messages, str)
                and len(messages) > 1
            ):
                if len(recipients) == len(messages):
                    for i, recipient in enumerate(recipients):
                        params[f"to[{recipient}]"] = messages[i]
                else:
                    raise ValueError(
                        "Sequences of recipients and messages "
                        "must be of the same length"
                    )
            elif (  # One message for all recipients
                isinstance(messages, str)
                or isinstance(messages, Sequence)
                and len(messages) == 1
            ):
                if isinstance(messages, Sequence) and not isinstance(messages, str):
                    messages = messages[0]
                params["to"] = ",".join(recipients)
                params["msg"] = messages
            else:
                raise ValueError(
                    "Messages must be str (message) " "or sequence of str (messages)"
                )
        elif (  # Only one recipient
            isinstance(recipients, str)
            or isinstance(recipients, Sequence)
            and len(recipients) == 1
        ):
            if (
                isinstance(messages, str)
                or isinstance(messages, Sequence)
                and len(messages) == 1
            ):
                if isinstance(messages, Sequence) and not isinstance(messages, str):
                    messages = messages[0]
                params["to"] = recipients
                params["msg"] = messages
            else:
                raise ValueError(
                    "If there is only one recipient, "
                    "then there can be no more than one message"
                )
        else:
            raise ValueError(
                "Recipients must be str (phone number) "
                "or sequence of str (phone numbers)"
            )

        params.update(kwargs)

        response = await self._request("sms/cost", params)

        return SMSruSmsCostResponse(**response.json())

    async def balance(self) -> SMSruWithBalance:
        """
        Get account balance

        :return: sms.ru response
        :rtype: SMSruWithBalance
        """
        response = await self._request("my/balance")

        return SMSruWithBalance(**response.json())

    async def limit(self) -> SMSruLimit:
        """
        Get account sms limits

        :return: sms.ru response
        :rtype: SMSruLimit
        """
        response = await self._request("my/limit")

        return SMSruLimit(**response.json())

    async def free_limit(self) -> SMSruFreeLimit:
        """
        Get account free sms limit

        :return: sms.ru response
        :rtype: SMSruFreeLimit
        """
        response = await self._request("my/free")

        return SMSruFreeLimit(**response.json())

    async def senders(self) -> SMSruSenders:
        """
        Get approved senders

        :return: sms.ru response
        :rtype: SMSruSenders
        """
        response = await self._request("my/senders")

        return SMSruSenders(**response.json())

    async def check_auth(
        self, api_id: str = None, login: str = None, password: str = None
    ) -> SMSruBase:
        """
        Check username/password or app_id for validity.
        Be sure to specify api_id or login and password

        :param api_id: api_id
        :type api_id: ``str``
        :param login: login (phone number in 79996662244 format)
        :type login: ``str``
        :param password: password
        :type password: ``str``
        :return: sms.ru response
        :rtype: SMSruBase
        """
        if api_id:
            params = {"api_id": api_id}
        elif login and password:
            params = {"login": login, "password": password}
        else:
            raise ValueError("You must provide api_id or login and password")

        response = await self.client.get(
            f"https://{self._domain}/auth/check", params=params
        )

        return SMSruBase(**response.json())

    async def stoplist(self) -> SMSruStoplist:
        response = await self._request("stoplist/get")

        return SMSruStoplist(**response.json())
